/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <iostream>

#include <o2scl/funct.h>
#include <o2scl/table.h>
#include <o2scl/eqi_deriv.h>
#include <o2scl/interp.h>
#include <o2scl/base_ioc.h>

/*
  A simple example for showing computation of derivatives with
  interpolation from the table class and using the eqi_deriv class.
  This also shows simple I/O with the table.
*/

using namespace std;
using namespace o2scl;

int main(void) {
  size_t bign=30, i;
  
  cout.precision(6);
  cout.setf(ios::scientific);

  ovector x(bign), y(bign), dydx(bign), dydx2(bign);

  // We create some data, y=exp(x).
  for(i=0;i<bign;i++) {
    x[i]=((double)i)/3.0;
    y[i]=exp(x[i]);
  }

  // Compute dydx using derivative formulas for equally-spaced abscissas
  eqi_deriv<void *,funct<void *> > ei;
  ei.deriv_array(bign,1.0/3.0,y,dydx);

  // Compute the derivative using the interpolation from o2scl_interp
  cspline_interp<ovector_view> it;
  o2scl_interp_vec<> gi(it,bign,x,y);
  for(i=0;i<bign;i++) {
    dydx2[i]=gi.deriv(x[i]);
  }

  // Create a data table 
  table ta(100);
  // Give the column names
  ta.line_of_names("x y dydx dydx2 err err2");

  cout << "x            y            dydx         dydx'"
       << "        err          err'" << endl;
  for(i=0;i<bign;i++) {
    cout << x[i] << " " << y[i] << " " << dydx[i] << " " << dydx2[i] << " ";
    cout << fabs(y[i]-dydx[i])/y[i] << " " << fabs(y[i]-dydx2[i])/y[i] << endl;
    // Add the data to the table as well...
    double line[6]={x[i],y[i],dydx[i],dydx2[i],
		    fabs(y[i]-dydx[i])/y[i],fabs(y[i]-dydx2[i])/y[i]};
    ta.line_of_data(6,line);
  }
  
  // Create a collection, the main I/O facility
  base_ioc bio;
  collection co;
  // Output results to file
  co.out_one("ex_diff.out","table","ex_diff",&ta);

  return 0;
}
  

