#include <iostream>
#include <o2scl/base_ioc.h>
#include <o2scl/table.h>
#include <o2scl/collection.h>
#include <o2scl/constants.h>

using namespace std;
using namespace o2scl;

int main(void) {

  cout.setf(ios::scientific);
  
  // Create a table with two columns. It is usually best to
  // specify in the constructor, the number of rows you will 
  // need before hand, i.e. table dat(100), but we do not
  // do so here for demonstration purposes.
  table dat;
  dat.line_of_names("x y");
  
  for(int i=0;i<11;i++) {
    dat.set("x",i,((double)i)/10.0);
    dat.set("y",i,sin(dat.get("x",i)));
  }
  
  // We demonstrate interpolation and differentiation and 
  // compare with the exact answers
  cout << " Result        Expected" << endl;
  cout << "----------------------------" << endl;
  cout << " " << dat.interp("y",0.5,"x") << "  " << asin(0.5) << endl;
  cout << " " << dat.deriv("x",0.5,"y") << "  " << cos(0.5) << endl;
  cout << dat.deriv2("x",0.5,"y") << " " << -sin(0.5) << endl;
  cout << endl;

  dat.new_column("cx");
  dat.new_column("sx");
  for(size_t i=0;i<dat.get_nlines();i++) {
    dat["sx"][i]=dat["y"][i];
    dat["cx"][i]=sqrt(1.0-pow(dat["sx"][i],2.0));
  }

  dat.add_constant("pi",acos(-1.0));

  dat.new_column("s2");
  for(size_t i=0;i<dat.get_nlines();i++) {
    dat["s2"][i]=sin(dat["x"][i]*acos(-1.0)/2.0);
  }

  // We get approximate derivatives for an entire column
  dat.deriv("x","sx","cx2");

  base_ioc bio;

  // We output this table to std::cout and also to a file
  collection co;
  cout.setf(ios::scientific);
  text_out_file tout(&cout);
  co.out_one(&tout,"table","dat",&dat);
  
  return 0;
}
