/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/misc.h>

using namespace std;
using namespace o2scl;

std::string o2scl::binary_to_hex(std::string s) {
  std::string t="";
  char nums[16]={'0','1','2','3','4','5','6','7',
		 '8','9','A','B','C','D','E','F'};
  for(size_t i=0;i<s.length();i++) {
    bool found=false;
    if (s[i]=='1' || s[i]=='0') {
      if (i+1<s.length() && (s[i+1]=='1' || s[i+1]=='0')) {
	if (i+2<s.length() && (s[i+2]=='1' || s[i+2]=='0')) {
	  if (i+3<s.length() && (s[i+3]=='1' || s[i+3]=='0')) {
	    found=true;
	    int cnt=0;
	    if (s[i]=='1') cnt+=8;
	    if (s[i+1]=='1') cnt+=4;
	    if (s[i+2]=='1') cnt+=2;
	    if (s[i+3]=='1') cnt++;
	    t+=nums[cnt];
	    i+=3;
	  }
	}
      }
    }
    if (found==false) {
      t+=s[i];
    }
  }
  return t;
}

double o2scl::fermi_function(double E, double mu, double T, double limit) {
  double ret, x=(E-mu)/T;
  
  if (x>limit) {
    ret=0.0;
  } else if (x<-limit) {
    ret=1.0;
  } else {
    ret=1.0/(1.0+exp(x));
  }
  return ret;
}

void o2scl::screenify(const string *in_cols, int nin, 
		    string *&outc, int &nout, int max_size) {
  int i,j,lmax,itemp;
  string *in_spaces=new string[nin];
  
  // Determine size of largest string
  lmax=0;
  for(i=0;i<nin;i++) {
    if (lmax<((int)in_cols[i].size())) lmax=in_cols[i].size();
  }

  // Pad with spaces
  for(i=0;i<nin;i++) {
    itemp=in_cols[i].size();
    in_spaces[i]=in_cols[i];
    for(j=0;j<lmax+1-itemp;j++) {
      in_spaces[i]+=' ';
    }
  }

  // Determine number of rows and columns
  int row, col;
  col=max_size/(lmax+1);
  if (col==0) col=1;
  if (nin/col*col==nin) row=nin/col;
  else row=nin/col+1;

  // Create outc
  outc=new string[row];
  for(i=0;i<row;i++) {
    for(j=0;j<col;j++) {
      if (i+j*row<nin) {
	outc[i]+=in_spaces[i+j*row];
      }
    }
  }
  nout=row;

  delete[] in_spaces;

  return;
  
}

int o2scl::count_words(string str) {
  string st;
  istringstream *is=new istringstream(str.c_str());
  int ctr=0;
  while((*is) >> st) ctr++;
  delete is;

  return ctr;
}
