/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GSL_INTE_QAGIL_H
#define O2SCL_GSL_INTE_QAGIL_H

#include <o2scl/inte.h>
#include <o2scl/gsl_inte_qags.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** \brief Integrate a function from \f$ -\infty \f$ to 
      \f$ \mathrm{b} \f$ (GSL)
  */
  template<class param_t, class func_t> class gsl_inte_qagil : 
  public gsl_inte_transform<param_t,func_t> {

#ifndef DOXYGEN_INTERNAL      

  protected:

    /// Store the upper limit
    double lb;

#endif

  public:
    
    /** 
	\brief Integrate function \c func from \f$ -\infty \f$ to \c b.

	The value given in \c a is ignored.
     */
    virtual double integ(func_t &func, double a, double b, param_t &pa) {
      double res, err;
      integ_err(func,a,b,pa,res,err);
      this->interror=err;
      return res;
    }
    
    /** 
	\brief Integrate function \c func from \f$ -\infty \f$ to \c b
	and place the result in \c res and the error in \c err2

	The value given in \c a is ignored.
    */
    virtual int integ_err(func_t &func, double a, double b, 
			  param_t &pa, double &res, double &err2) {
      
      lb=b;

      double fv1[8], fv2[8];
	
      int status=qags(func,8,o2scl_inte_qag_coeffs::qk15_xgk,
		       o2scl_inte_qag_coeffs::qk15_wg,
		       o2scl_inte_qag_coeffs::qk15_wgk,
		       fv1,fv2,0.0,1.0,this->tolx,this->tolf,
		       this->wkspace,&res,&err2,pa);
      return status;
	
    }

  protected:
      
    /// Transform to \f$ t \in (0,1] \f$
    virtual double transform(func_t &func, double t, param_t &pa) {
      double x=lb-(1-t)/t, y;
      func(x,y,pa);
      return y/t/t;
    }
      
  };
  
#ifndef DOXYGENP
}
#endif

#endif
