/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_inte_qagiu.h>

using namespace std;
using namespace o2scl;

int testfun(double x, double &y, void *&pa) {
  y=exp(-x*x);
  return 0;
}

int testfun2(double x, double &y, void *&pa) {
  y=sin(1.0/(x+0.01))*pow(x+0.01,-2.0);
  return 0;
}

double f(double x, void *params) {
  double f=sin(1.0/(x+0.01))*pow(x+0.01,-2.0);
  return f;
}

int main(void) {
  test_mgr t;
  t.set_output_level(2);
  funct_fptr<void *> tf(&testfun), tf2(&testfun2);
  
  gsl_inte_qagiu<void *, funct_fptr<void *> > it;

  double ans, exact;
  void *vp=NULL;

  cout.setf(ios::scientific);
  cout.precision(10);
  
  exact=0.5*sqrt(acos(-1.0));
  ans=it.integ(tf,0.0,0.0,vp);
  t.test_rel(ans,exact,1.0e-8,"qagiu test");

  exact=1.0-cos(100.0/101.0);
  ans=it.integ(tf2,1.0,0.0,vp);
  t.test_rel(ans,exact,1.0e-8,"qagiu test 2");

  exact=1.0-cos(100.0);
  ans=it.integ(tf2,0.0,0.0,vp);
  t.test_rel(ans,exact,1.0e-8,"qagiu test 2");

  // Compare with GSL and make sure we get the same result.  This
  // doesn't always agree exactly, but I'm pretty sure this is
  // numerical precision errors. In any case, it's well within the
  // uncertainty given by the routine
  {
    gsl_integration_workspace *w=gsl_integration_workspace_alloc(1000);
    
    double result, error;
    
    gsl_function F;
    F.function=&f;
    F.params=0;
    
    gsl_integration_qagiu(&F,0,0,1e-7,1000,w,&result,&error); 

    t.test_rel(ans,result,1.0e-11,"GSL vs. O2scl");

    std::string s1=double_to_ieee_string(&ans);
    std::string s2=double_to_ieee_string(&result);
    cout << s1 << endl;
    cout << s2 << endl;
    
    gsl_integration_workspace_free(w);
  }

  t.report();
  return 0;
}
