/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GSL_INTE_QAWS_H
#define O2SCL_GSL_INTE_QAWS_H

#include <o2scl/inte.h>
#include <o2scl/gsl_inte_qag_b.h>

#ifndef DOXYGENP
namespace o2scl {
#endif
  
  /** 
      \brief QAWS integration (GSL)

      \note This is unfinished.
      
      \todo Finish this!
  */
  template<class param_t, class func_t> class gsl_inte_qaws : 
  public gsl_inte_cheb<param_t,func_t> {
    
  public:

    gsl_inte_qaws() {
    }

    virtual ~gsl_inte_qaws() {}
      
    /** \brief Integrate function \c func from \c a to \c b.
     */
    virtual double integ(func_t &func, double a, double b, param_t &pa) {
      double res, err;
      integ_err(func,a,b,pa,res,err);
      this->interror=err;
      return res;
    }
    
    /// The singularity
    double s;

    /** \brief Integrate function \c func from \c a to \c b and place
	the result in \c res and the error in \c err
    */
    virtual int integ_err(func_t &func, double a, double b, 
			  param_t &pa, double &res, double &err2) {

      int status=qaws(func,a,b,s,this->tolx,this->tolf,this->wkspace,
		      &res,&err2,pa);
      
      return status;
      
    }

#ifndef DOXYGEN_INTERNAL

  protected:

    /** 
	\brief Desc
    */
    int qaws(func_t &func, const double a, const double b, const double c,
	     const double epsabs, const double epsrel, const size_t limit,
	     double *result, double *abserr, param_t &pa) {
      double area, errsum;
      double result0, abserr0;
      double tolerance;
      size_t iteration = 0;
      int roundoff_type1 = 0, roundoff_type2 = 0, error_type = 0;

      /* Initialize results */

      initialise (workspace, a, b);

      *result = 0;
      *abserr = 0;

      if (limit > workspace->limit) {
	GSL_ERROR ("iteration limit exceeds available workspace", 
		   GSL_EINVAL) ;
      }

      if (b <= a) {
	GSL_ERROR ("limits must form an ascending sequence, a < b", 
		   GSL_EINVAL) ;
      }
      
      if (epsabs <= 0 && (epsrel < 50 * GSL_DBL_EPSILON || 
			  epsrel < 0.5e-28)) {
	GSL_ERROR ("tolerance cannot be acheived with given epsabs and epsrel",
		   GSL_EBADTOL);
      }
      
      /* perform the first integration */

      {
	double area1, area2;
	double error1, error2;
	int err_reliable1, err_reliable2;
	double a1 = a;
	double b1 = 0.5 * (a + b);
	double a2 = b1;
	double b2 = b;

	qc25s (f, a, b, a1, b1, t, &area1, &error1, &err_reliable1);
	qc25s (f, a, b, a2, b2, t, &area2, &error2, &err_reliable2);
    
	if (error1 > error2)
	  {
	    append_interval (workspace, a1, b1, area1, error1);
	    append_interval (workspace, a2, b2, area2, error2);
	  }
	else
	  {
	    append_interval (workspace, a2, b2, area2, error2);
	    append_interval (workspace, a1, b1, area1, error1);
	  }
    
	result0 = area1 + area2;
	abserr0 = error1 + error2;
      }

      /* Test on accuracy */

      tolerance = GSL_MAX_DBL (epsabs, epsrel * fabs (result0));

      /* Test on accuracy, use 0.01 relative error as an extra safety
	 margin on the first iteration (ignored for subsequent iterations) */

      if (abserr0 < tolerance && abserr0 < 0.01 * fabs(result0))
	{
	  *result = result0;
	  *abserr = abserr0;

	  return GSL_SUCCESS;
	}
      else if (limit == 1)
	{
	  *result = result0;
	  *abserr = abserr0;

	  GSL_ERROR ("a maximum of one iteration was insufficient", 
		     GSL_EMAXITER);
	}

      area = result0;
      errsum = abserr0;

      iteration = 2;

      do
	{
	  double a1, b1, a2, b2;
	  double a_i, b_i, r_i, e_i;
	  double area1 = 0, area2 = 0, area12 = 0;
	  double error1 = 0, error2 = 0, error12 = 0;
	  int err_reliable1, err_reliable2;

	  /* Bisect the subinterval with the largest error estimate */

	  retrieve (workspace, &a_i, &b_i, &r_i, &e_i);

	  a1 = a_i; 
	  b1 = 0.5 * (a_i + b_i);
	  a2 = b1;
	  b2 = b_i;

	  qc25s (f, a, b, a1, b1, t, &area1, &error1, &err_reliable1);
	  qc25s (f, a, b, a2, b2, t, &area2, &error2, &err_reliable2);

	  area12 = area1 + area2;
	  error12 = error1 + error2;

	  errsum += (error12 - e_i);
	  area += area12 - r_i;

	  if (err_reliable1 && err_reliable2)
	    {
	      double delta = r_i - area12;

	      if (fabs (delta) <= 1.0e-5 * fabs (area12) && 
		  error12 >= 0.99 * e_i)
		{
		  roundoff_type1++;
		}
	      if (iteration >= 10 && error12 > e_i)
		{
		  roundoff_type2++;
		}
	    }

	  tolerance = GSL_MAX_DBL (epsabs, epsrel * fabs (area));

	  if (errsum > tolerance)
	    {
	      if (roundoff_type1 >= 6 || roundoff_type2 >= 20)
		{
		  error_type = 2;   /* round off error */
		}

	      /* set error flag in the case of bad integrand behaviour at
		 a point of the integration range */

	      if (subinterval_too_small (a1, a2, b2))
		{
		  error_type = 3;
		}
	    }

	  update (workspace, a1, b1, area1, error1, a2, b2, area2, error2);

	  retrieve (workspace, &a_i, &b_i, &r_i, &e_i);

	  iteration++;

	}
      while (iteration < limit && !error_type && errsum > tolerance);

      *result = sum_results (workspace);
      *abserr = errsum;

      if (errsum <= tolerance)
	{
	  return GSL_SUCCESS;
	}
      else if (error_type == 2)
	{
	  GSL_ERROR ("roundoff error prevents tolerance from being achieved",
		     GSL_EROUND);
	}
      else if (error_type == 3)
	{
	  GSL_ERROR 
	    ("bad integrand behavior found in the integration interval",
	     GSL_ESING);
	}
      else if (iteration == limit)
	{
	  GSL_ERROR ("maximum number of subdivisions reached", GSL_EMAXITER);
	}
      else
	{
	  GSL_ERROR ("could not integrate function", GSL_EFAILED);
	}

    }

    struct fn_qaws_params
    {
      gsl_function *function;
      double a;
      double b;
      gsl_integration_qaws_table *table;
    };

    double fn_qaws (double t, void *params);
    double fn_qaws_L (double x, void *params);
    double fn_qaws_R (double x, void *params);

    void
      compute_result (const double * r, const double * cheb12, const double * cheb24,
		      double * result12, double * result24);


    void
      qc25s (gsl_function * f, double a, double b, double a1, double b1,
	     gsl_integration_qaws_table * t,
	     double *result, double *abserr, int *err_reliable);

    void
      qc25s (gsl_function * f, double a, double b, double a1, double b1,
	     gsl_integration_qaws_table * t,
	     double *result, double *abserr, int *err_reliable)
      {
	gsl_function weighted_function;
	struct fn_qaws_params fn_params;
  
	fn_params.function = f;
	fn_params.a = a;
	fn_params.b = b;
	fn_params.table = t;

	weighted_function.params = &fn_params;
    
	if (a1 == a && (t->alpha != 0.0 || t->mu != 0))
	  {
	    double cheb12[13], cheb24[25];

	    double factor = pow(0.5 * (b1 - a1), t->alpha + 1.0);

	    weighted_function.function = &fn_qaws_R;

	    gsl_integration_qcheb (&weighted_function, a1, b1, cheb12, cheb24);

	    if (t->mu == 0)
	      {
		double res12 = 0, res24 = 0;
		double u = factor;

		compute_result (t->ri, cheb12, cheb24, &res12, &res24);

		*result = u * res24;
		*abserr = fabs(u * (res24 - res12));
	      }
	    else 
	      {
		double res12a = 0, res24a = 0;
		double res12b = 0, res24b = 0;

		double u = factor * log(b1 - a1);
		double v = factor;

		compute_result (t->ri, cheb12, cheb24, &res12a, &res24a);
		compute_result (t->rg, cheb12, cheb24, &res12b, &res24b);

		*result = u * res24a + v * res24b;
		*abserr = fabs(u * (res24a - res12a)) + fabs(v * (res24b - res12b));
	      }

	    *err_reliable = 0;

	    return;
	  }
	else if (b1 == b && (t->beta != 0.0 || t->nu != 0))
	  {
	    double cheb12[13], cheb24[25];
	    double factor = pow(0.5 * (b1 - a1), t->beta + 1.0);

	    weighted_function.function = &fn_qaws_L;

	    gsl_integration_qcheb (&weighted_function, a1, b1, cheb12, cheb24);

	    if (t->nu == 0)
	      {
		double res12 = 0, res24 = 0;
		double u = factor;

		compute_result (t->rj, cheb12, cheb24, &res12, &res24);

		*result = u * res24;
		*abserr = fabs(u * (res24 - res12));
	      }
	    else 
	      {
		double res12a = 0, res24a = 0;
		double res12b = 0, res24b = 0;

		double u = factor * log(b1 - a1);
		double v = factor;

		compute_result (t->rj, cheb12, cheb24, &res12a, &res24a);
		compute_result (t->rh, cheb12, cheb24, &res12b, &res24b);

		*result = u * res24a + v * res24b;
		*abserr = fabs(u * (res24a - res12a)) + fabs(v * (res24b - res12b));
	      }

	    *err_reliable = 0;

	    return;
	  }
	else
	  {
	    double resabs, resasc;

	    weighted_function.function = &fn_qaws;
  
	    gsl_integration_qk15 (&weighted_function, a1, b1, result, abserr,
				  &resabs, &resasc);

	    if (*abserr == resasc)
	      {
		*err_reliable = 0;
	      }
	    else 
	      {
		*err_reliable = 1;
	      }

	    return;
	  }

      }

    double
      fn_qaws (double x, void *params)
      {
	struct fn_qaws_params *p = (struct fn_qaws_params *) params;
	gsl_function *f = p->function;
	gsl_integration_qaws_table *t = p->table;

	double factor = 1.0;
  
	if (t->alpha != 0.0)
	  factor *= pow(x - p->a, t->alpha);

	if (t->beta != 0.0)
	  factor *= pow(p->b - x, t->beta);

	if (t->mu == 1)
	  factor *= log(x - p->a);

	if (t->nu == 1)
	  factor *= log(p->b - x);

	return factor * GSL_FN_EVAL (f, x);
      }

    double
      fn_qaws_L (double x, void *params)
      {
	struct fn_qaws_params *p = (struct fn_qaws_params *) params;
	gsl_function *f = p->function;
	gsl_integration_qaws_table *t = p->table;

	double factor = 1.0;
  
	if (t->alpha != 0.0)
	  factor *= pow(x - p->a, t->alpha);

	if (t->mu == 1)
	  factor *= log(x - p->a);

	return factor * GSL_FN_EVAL (f, x);
      }

    double
      fn_qaws_R (double x, void *params)
      {
	struct fn_qaws_params *p = (struct fn_qaws_params *) params;
	gsl_function *f = p->function;
	gsl_integration_qaws_table *t = p->table;

	double factor = 1.0;
  
	if (t->beta != 0.0)
	  factor *= pow(p->b - x, t->beta);

	if (t->nu == 1)
	  factor *= log(p->b - x);

	return factor * GSL_FN_EVAL (f, x);
      }


    void
      compute_result (const double * r, const double * cheb12, const double * cheb24,
		      double * result12, double * result24)
      {
	size_t i;
	double res12 = 0;
	double res24 = 0;
  
	for (i = 0; i < 13; i++)
	  {
	    res12 += r[i] * cheb12[i];
	  }
  
	for (i = 0; i < 25; i++)
	  {
	    res24 += r[i] * cheb24[i];
	  }
  
	*result12 = res12;
	*result24 = res24;
      }


    /// Desc
    virtual double transform(func_t &func, double x, param_t &pa)
    {
      double y;
      func(x,y,pa);
      return y/(x-s);
    }

#endif
  
    /// Return string denoting type ("gsl_inte_qaws")
    const char *type() { return "gsl_inte_qaws"; }
  
  };

#ifndef DOXYGENP
}
#endif

#endif
