/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#ifndef O2SCL_GSL_MMIN_SIMP_B_H
#define O2SCL_GSL_MMIN_SIMP_B_H

#include <iostream>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_multimin.h>
#include <gsl/gsl_blas.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Base routines for the GSL simplex minimizer
   */
  class gsl_mmin_simp_b {
  public:
    
    /// State type for GSL simplex minimizer
    typedef struct
    {
      /// The (n+1,n) matrix containing the simplex
      gsl_matrix *x1;
      /// The (n+1) function values at the simplex points
      gsl_vector *y1;
      /// Desc
      gsl_vector *ws1;
      /// Desc
      gsl_vector *ws2;
    }
    simp_state_t;
    
    /// Compute the center of the simplex and store in \c mp
    int nmsimplex_calc_center(const simp_state_t *state, gsl_vector *mp);
    
    /** 
	\brief Compute the size of the simplex
	
	Calculates simplex size as average sum of length of vectors
	from simplex center to corner points:
	
	\f$ (1/n) \sum || y - y_{\mathrm{middlepoint}} || \f$
    */
    double nmsimplex_size(simp_state_t *state);
    
  };


#ifndef DOXYGENP
}
#endif

#endif
