/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/multi_funct.h>
#include <o2scl/gsl_mmin_simp.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

double minfun(size_t n, const ovector_view &x, void *&pa) {
  double ret;
  ret=x[0]*x[0]+(x[1]-2.0)*(x[1]-2.0)+3.0;
  return(ret);
}

int main(void) {
  test_mgr t;
  t.set_output_level(2);
  double min=0.0, min2;
  ovector x(3);
  void *vp=0;
  gsl_mmin_simp<void *,multi_funct<void *> > g;

  cout.setf(ios::scientific);

  multi_funct_fptr_noerr<void *> mf(minfun);
  
  x[0]=1.0;
  x[1]=1.0;
  //g.verbose=1;
  g.print_simplex=1;
  int ret=g.mmin(2,x,min,vp,mf);
  cout << min << " " << x[0] << " " << x[1] << endl;
  t.test_abs(x[0],0.0,1.0e-4,"gsl_mmin_nmsimplex 1");
  t.test_rel(x[1],2.0,2.0e-4,"gsl_mmin_nmsimplex 2");
  g.verbose=0;
  cout << endl;

  ovector x1(5), x2(5);
  for(size_t i=0;i<5;i++) {
    x1[i]=0.0;
    x2[i]=1.0;
  }
  g.allocate(5);
  g.set(mf,vp,x1,x2);
  
  t.report();
  return 0;
}
 
