/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef MASS_FIT_H
#define MASS_FIT_H

#include <o2scl/constants.h>
#include <o2scl/multi_funct.h>
#include <o2scl/multi_min.h>
#include <o2scl/gsl_mmin_simp.h>
#include <o2scl/gsl_mmin_conp.h>
#include <o2scl/nuclear_mass.h>
#include <o2scl/nuclear_dist.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Fit a nuclear mass formula

      \todo Convert to a real fit with errors and covariance, etc.
   */
  class mass_fit {
  public:
  
    mass_fit();
    
    virtual ~mass_fit() {};
    
    /// If true, then only fit doubly-even nuclei (default false)
    bool even_even;
   
    /// Minimum proton number to fit (default 8)
    int minZ;
    
    /// Minimum neutron number to fit (default 8)
    int minN;

    /// Fit the nuclear mass formula
    virtual int fit(nuclear_mass_fit &n, double &res);
    
    /** 
	\brief Evaluate quality without fitting
     */
    virtual int eval(nuclear_mass &n, double &res);

    /** 
	\brief The default minimizer

	The value of def_mmin::ntrial is automatically multiplied by
	10 in the constructor because the minimization frequently
	requires more trials than the default.
     */
    gsl_mmin_simp<void *,multi_funct<void *> > def_mmin;

    /// Change the minimizer for use in the fit
    int set_mmin(multi_min<void *,multi_funct<void *> > &umm) {
      mm=&umm;
      return 0;
    }

    /** \brief The default distribution of nuclei to fit (defaults 
	to all nuclei in \ref def_exp_mass)
    */
    full_dist def_dist;

    /// Set the distribution of nuclei to fit
    int set_dist(nuclear_dist &und) {
      nd=&und;
      return 0;
    }

    /// The default experimental nuclear mass object
    ame_mass def_exp_mass;

    /// Set the experimental values to fit to
    int set_masses(nuclear_mass &uexp) {
      exp=&uexp;
      return 0;
    }

  protected:

#ifndef DOXYGENP
    
    /** 
	\brief The function to minimize
     */
    virtual int min_fun(size_t nv, const ovector_view &x, double &y, 
			void *&vp);

    /// The pointer to the minimizer
    multi_min<void *,multi_funct<void *> > *mm;
    
    /// Temporary store a pointer to the nuclear mass formula to fit to
    nuclear_mass_fit *nmf;
    
    /// A pointer to the experimental nuclei
    nuclear_mass *exp;

    /// A pointer to the nuclear distribution
    nuclear_dist *nd;

#endif

  };

#ifndef DOXYGENP
}
#endif

#endif
