/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/ode_funct.h>
#include <o2scl/nonadapt_step.h>

using namespace std;
using namespace o2scl;

int derivs(double x, size_t nv, const ovector_view &y, ovector_view &dydx, 
	   void *&pa) {
  dydx[0]=y[0];
  return 0;
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  nonadapt_step<void *,ode_funct_fptr<void *>,ovector_view,ovector,
    ovector_alloc> na;
  double x, dx;
  ovector y(1);
  int i;
  void *vpx=NULL;
  
  ode_funct_fptr<void *,ovector_view> od(derivs);
  
  cout.setf(ios::scientific);

  x=1.0;
  y[0]=1.0;
  dx=0.1;
  cout << "x            y(calc)      y(exact)     diff" << endl;
  cout << x << " " << y[0] << " " << exp(x-1.0) << " " 
       << fabs(y[0]-exp(x-1.0))/exp(x-1.0) << endl;
  for(i=1;i<=40;i++) {
    na.astep(x,dx,5.0,1,y,vpx,od);
    t.test_rel(y[0],exp(x-1.0),2.0e-3,"y_calculated-y_exact");
    cout << x << " " << y[0] << " " << exp(x-1.0) << " " 
	 << fabs(y[0]-exp(x-1.0))/exp(x-1.0) << endl;
  }

  t.report();

  return 0;
}
