#include <o2scl/ode_it_solve.h>

#include <o2scl/coord.h>
#include <o2scl/compcol.h>
#include <o2scl/diagpre.h>
#include <o2scl/icpre.h>
#include <o2scl/ilupre.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_iml;

class fc1 {
  
public:

  double left(size_t ieq, double x, ovector_view &yleft) {
    return yleft[1]-1.0;
  }
  
  double right(size_t ieq, double x, ovector_view &yright) {
    return yright[0]-2.0;
  }
  
  double derivs(size_t ieq, double x, ovector_view &y) {
    if (ieq==0) return y[0]+y[1];
    return y[0];
  }
  
};

class fc2 {
  
public:
  
  double left(size_t ieq, double x, ovector_view &yleft) {
    return yleft[0]-1.0;
  }
  
  double right(size_t ieq, double x, ovector_view &yright) {
    return yright[1]-2.0;
  }
  
  double derivs(size_t ieq, double x, ovector_view &y) {
    if (ieq==1) return y[0]+y[1];
    return y[1];
  }
  
};

class fc3 {
  
public:
  
  double left(size_t ieq, double x, ovector_view &yleft) {
    if (ieq==0) return yleft[0]-1.0;
    return yleft[1]*yleft[1]+yleft[2]*yleft[2]-2.0;
  }
  
  double right(size_t ieq, double x, ovector_view &yright) {
    return yright[1]-3.0;
  }
  
  double derivs(size_t ieq, double x, ovector_view &y) {
    if (ieq==1) return y[0]+y[1];
    else if (ieq==2) return y[0]+y[2];
    return y[1];
  }
  
};

class fc4 {
  
public:
  
  double left(size_t ieq, double x, ovector_view &yleft) {
    return yleft[0];
  }
  
  double right(size_t ieq, double x, ovector_view &yright) {
    if (ieq==0) return yright[0]-1.0;
    return yright[1]*yright[1]-yright[2]*yright[2]-2.0;
  }
  
  double derivs(size_t ieq, double x, ovector_view &y) {
    if (ieq==1) return y[0]+y[1];
    else if (ieq==2) return y[0]+y[2];
    return y[1];
  }
  
};

class fc4u {
  
public:
  
  double left(size_t ieq, double x, uvector_view &yleft) {
    return yleft[0];
  }
  
  double right(size_t ieq, double x, uvector_view &yright) {
    if (ieq==0) return yright[0]-1.0;
    return yright[1]*yright[1]-yright[2]*yright[2]-2.0;
  }
  
  double derivs(size_t ieq, double x, uvector_view &y) {
    if (ieq==1) return y[0]+y[1];
    else if (ieq==2) return y[0]+y[2];
    return y[1];
  }
  
};

int main(void) {

  double zx[3][3];
  //  array_row<double,double [3][3]> za(zx,1);
  
  ode_it_solve<ode_it_funct<ovector_view>,ovector_view,
    omatrix_view,omatrix_row,ovector_view,omatrix_view> oit;
  
  gsl_HH_solver ghh;
  oit.set_solver(ghh);

  fc1 f1;
  fc2 f2;
  fc3 f3;
  fc4 f4;
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);
  
  ode_it_funct_mfptr<fc1> ofm1d(&f1,&fc1::derivs);
  ode_it_funct_mfptr<fc1> ofm1l(&f1,&fc1::left);
  ode_it_funct_mfptr<fc1> ofm1r(&f1,&fc1::right);
  ode_it_funct_mfptr<fc2> ofm2d(&f2,&fc2::derivs);
  ode_it_funct_mfptr<fc2> ofm2l(&f2,&fc2::left);
  ode_it_funct_mfptr<fc2> ofm2r(&f2,&fc2::right);
  ode_it_funct_mfptr<fc3> ofm3d(&f3,&fc3::derivs);
  ode_it_funct_mfptr<fc3> ofm3l(&f3,&fc3::left);
  ode_it_funct_mfptr<fc3> ofm3r(&f3,&fc3::right);
  ode_it_funct_mfptr<fc4> ofm4d(&f4,&fc4::derivs);
  ode_it_funct_mfptr<fc4> ofm4l(&f4,&fc4::left);
  ode_it_funct_mfptr<fc4> ofm4r(&f4,&fc4::right);

  // Test on a small grid
  
  ovector x(5);
  omatrix y(5,2);
  for(int i=0;i<5;i++) {
    x[i]=((double)i)/4.0;
    y[i][0]=2.0*x[i];
    y[i][1]=1.0+x[i]/2;
  }
  
  omatrix A1(10,10);
  ovector rhs1(10), dy1(10);
  oit.solve(5,2,1,x,y,ofm1d,ofm1l,ofm1r,A1,rhs1,dy1);

  // Larger grid

  ovector x2(11);
  omatrix y2(11,2);
  for(int i=0;i<11;i++) {
    x2[i]=((double)i)/10.0;
    y2[i][0]=2.0*x2[i];
    y2[i][1]=1.0+x2[i]/2;
  }
  
  omatrix A2(22,22);
  ovector rhs2(22), dy2(22);
  oit.solve(11,2,1,x2,y2,ofm1d,ofm1l,ofm1r,A2,rhs2,dy2);

  // Compare to exact solution

  double s5=sqrt(5.0), sol1, sol2;
  for(int kk=0;kk<11;kk++) {
    double z=x2[kk];
    sol1=2.0*exp(0.5*(z-s5*z))/
      (5.0*sqrt(exp(1.0))+(5.0+s5)*exp(0.5+s5)-
       sqrt(5.0*exp(1.0)))*
      (-(-5.0+s5)*exp(s5/2.0)-s5*exp(0.5+s5)+
       (5.0+s5)*exp(0.5*s5*(1.0+2.0*z))+
       s5*exp(0.5+s5*z));
    sol2=exp(0.5*(z-s5*z))/
      (5.0*sqrt(exp(1.0))+(5.0+s5)*exp(0.5+s5)-
       sqrt(5.0*exp(1.0)))*
      (-4.0*s5*exp(s5/2.0)+(5.0+s5)*exp(0.5+s5)+
       4.0*s5*exp(0.5*s5*(1.0+2.0*z))-
       (-5.0+s5)*exp(0.5+s5*z));
    if (kk==0) {
      t.test_rel(y2[kk][0],sol1,7.0e-1,"y20");
      t.test_rel(y2[kk][1],sol2,7.0e-1,"y21");
    } else {
      t.test_rel(y2[kk][0],sol1,5.0e-2,"y20");
      t.test_rel(y2[kk][1],sol2,5.0e-2,"y21");
    }
  }

  // Try reversing the order of the equations

  ovector x3(11);
  omatrix y3(11,2);
  for(int i=0;i<11;i++) {
    x3[i]=((double)i)/10.0;
    y3[i][0]=1.0+x3[i]/2;
    y3[i][1]=2.0*x3[i];
  }
  
  omatrix A3(22,22);
  ovector rhs3(22), dy3(22);
  oit.solve(11,2,1,x3,y3,ofm2d,ofm2l,ofm2r,A3,rhs3,dy3);

  for(int kk=0;kk<11;kk++) {
    double z=x3[kk];
    sol1=exp(0.5*(z-s5*z))/
      (5.0*sqrt(exp(1.0))+(5.0+s5)*exp(0.5+s5)-
       sqrt(5.0*exp(1.0)))*
      (-4.0*s5*exp(s5/2.0)+(5.0+s5)*exp(0.5+s5)+
       4.0*s5*exp(0.5*s5*(1.0+2.0*z))-
       (-5.0+s5)*exp(0.5+s5*z));
    sol2=2.0*exp(0.5*(z-s5*z))/
      (5.0*sqrt(exp(1.0))+(5.0+s5)*exp(0.5+s5)-
       sqrt(5.0*exp(1.0)))*
      (-(-5.0+s5)*exp(s5/2.0)-s5*exp(0.5+s5)+
       (5.0+s5)*exp(0.5*s5*(1.0+2.0*z))+
       s5*exp(0.5+s5*z));
    if (kk==0) {
      t.test_rel(y3[kk][0],sol1,7.0e-1,"y30");
      t.test_rel(y3[kk][1],sol2,7.0e-1,"y31");
    } else {
      t.test_rel(y3[kk][0],sol1,5.0e-2,"y30");
      t.test_rel(y3[kk][1],sol2,5.0e-2,"y31");
    }
  }
  
  // More difficult example 1

  ovector x4(5);
  omatrix y4(5,3);
  for(int i=0;i<5;i++) {
    x4[i]=((double)i)/4.0;
    y4[i][0]=1.0+x4[i]+1.0;
    y4[i][1]=3.0*x4[i];
    y4[i][2]=-0.1*x4[i]-1.4;
  }
  
  omatrix A4(15,15);
  ovector rhs4(15), dy4(15);
  oit.solve(5,3,2,x4,y4,ofm3d,ofm3l,ofm3r,A4,rhs4,dy4);

  t.test_rel(y4[0][0],1.0,1.0e-3,"y4a");
  t.test_rel(y4[4][0],2.30666,4.0e-3,"y4b");
  t.test_rel(y4[0][1],0.259509,1.0e-1,"y4c");
  t.test_rel(y4[4][1],3.0,1.0e-3,"y4d");
  t.test_rel(y4[0][2],-1.3902,4.0e-3,"y4e");
  t.test_rel(y4[4][2],-1.48437,3.0e-2,"y4f");
  
  // More difficult example 2
  
  ovector x5(5);
  omatrix y5(5,3);
  for(int i=0;i<5;i++) {
    x5[i]=((double)i)/4.0;
    y5[i][0]=x5[i];
    y5[i][1]=1.5*x5[i]+0.5;
    y5[i][2]=-x5[i]-0.6;
  }
  
  omatrix A5(15,15);
  ovector rhs5(15), dy5(15);
  //ghh.debug=true;
  oit.solve(5,3,1,x5,y5,ofm4d,ofm4l,ofm4r,A5,rhs5,dy5);
  
  t.test_rel(y5[0][0],0.0,1.0e-3,"y5a");
  t.test_rel(y5[4][0],1.0,1.0e-3,"y5b");
  t.test_rel(y5[0][1],0.496445,4.0e-2,"y5c");
  t.test_rel(y5[4][1],1.88562,1.0e-2,"y5d");
  t.test_rel(y5[0][2],-0.656063,1.0e-3,"y5e");
  t.test_rel(y5[4][2],-1.24722,1.0e-2,"y5f");
  
  // Redo more difficult example 2

  fc4u f4u;
  ode_it_funct_mfptr<fc4u,uvector_view> ufm4d(&f4u,&fc4u::derivs);
  ode_it_funct_mfptr<fc4u,uvector_view> ufm4l(&f4u,&fc4u::left);
  ode_it_funct_mfptr<fc4u,uvector_view> ufm4r(&f4u,&fc4u::right);
  
  ode_it_solve<ode_it_funct<uvector_view>,uvector_view,
    umatrix_view,umatrix_row,uvector_view,CompCol_Mat> oit2;
 
  /* 
     iml_solver<CompCol_Mat,CompCol_ILU_Pre> iml;
     oit2.set_solver(iml);
     
     uvector x6(5);
     umatrix y6(5,3);
     for(int i=0;i<5;i++) {
     x6[i]=((double)i)/4.0;
     y6[i][0]=x5[i];
     y6[i][1]=1.5*x5[i]+0.5;
     y6[i][2]=-x5[i]-0.6;
     }
  
     ode_it_make_Coord oimc;
     Coord_Mat *A6=oimc.make(5,3,1);
     uvector rhs6(15), dy6(15);

     cout << endl;
     cout << "Matrix form: " << endl;
     for(size_t i=0;i<15;i++) {
     for(size_t j=0;j<15;j++) {
     A6->set(i,j,1.0);
     if ((*A6)(i,j)>0.5) cout << "1 ";
     else cout << ". ";
     A6->set(i,j,0.0);
     }
     cout << endl;
     }
     cout << endl;
  
     CompCol_Mat A7(*A6);
     cout << "Matrix form: " << endl;
     for(size_t i=0;i<15;i++) {
     for(size_t j=0;j<15;j++) {
     A7.set(i,j,1.0);
     if (A7(i,j)>0.5) cout << "1 ";
     else cout << ". ";
     A7.set(i,j,0.0);
     }
     cout << endl;
     }
     cout << endl;
  
     oit2.solve(5,3,1,x6,y6,ufm4d,ufm4l,ufm4r,A7,rhs6,dy6);
  
     t.test_rel(y6[0][0],0.0,1.0e-3,"y6a");
     t.test_rel(y6[4][0],1.0,1.0e-3,"y6b");
     t.test_rel(y6[0][1],0.496445,4.0e-2,"y6c");
     t.test_rel(y6[4][1],1.88562,1.0e-2,"y6d");
     t.test_rel(y6[0][2],-0.656063,1.0e-3,"y6e");
     t.test_rel(y6[4][2],-1.24722,1.0e-2,"y6f");

  */
  
  t.report();

  return 0;
}
