/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/omatrix_tlate.h>
#include <o2scl/ode_iv_solve.h>
#include <o2scl/test_mgr.h>
#include <o2scl/ode_funct.h>

using namespace std;
using namespace o2scl;

int derivs(double x, size_t nv, const ovector_view &y, ovector_view &dydx, 
	   void *&pa) {
  dydx[0]=y[0];
  return 0;
}

int main(void) {
  int i;
  ovector y, dydx, yout, yerr;
  double x, dx=1.0e-1;
  y.allocate(2);
  dydx.allocate(2);
  yout.allocate(2);
  yerr.allocate(2);
  void *vp=NULL;
  test_mgr t;
  t.set_output_level(1);

  cout.setf(ios::scientific);

  ode_funct_fptr<void *> od(derivs);
  double hdid, hnext;

  cout << endl;
  ode_iv_solve<void *,ode_funct<void *>,ovector_view,ovector,
    ovector_alloc> ivs;
  y[0]=1.0;
  ovector yend(1);
  
  ivs.solve_final_value(1.0,5.0,4.0,1,y,yend,vp,od);
  
  cout << yend[0] << " " << fabs(yend[0]-exp(4.0))/exp(4.0) << endl;
  t.test_rel(yend[0],exp(4.0),1.0e-6,"final value");
  
  cout << endl;
  ovector xgrid(11);
  omatrix ygrid(11,1);
  int ngrid=11;
  y[0]=1.0;
  
  ivs.solve_grid(1.0,5.0,4.0,1,y,ngrid,xgrid,ygrid,vp,od);

  for(i=0;i<ngrid;i++) {
    cout << xgrid[i] << " " << ygrid[i][0] << " " 
	 << exp(xgrid[i]-1.0) << " "
	 << fabs(ygrid[i][0]-exp(xgrid[i]-1.0))/exp(xgrid[i]-1.0) 
	 << endl;
    t.test_rel(ygrid[i][0],exp(xgrid[i]-1.0),1.0e-6,"grid");
  }
  ygrid.free();
  
  cout << endl;
  size_t ng2=31;
  ovector agrid(ng2);
  omatrix bgrid(ng2,1);
  y[0]=1.0;

  ivs.solve_table(1.0,5.0,4.0,1,y,ng2,agrid,bgrid,vp,od);
  cout << ng2 << endl;

  for(i=0;i<((int)ng2);i++) {
    cout << agrid[i] << " " << bgrid[i][0] << " " 
	 << exp(agrid[i]-1.0) << " "
	 << fabs(bgrid[i][0]-exp(agrid[i]-1.0))/exp(agrid[i]-1.0) 
	 << endl;
    t.test_rel(bgrid[i][0],exp(agrid[i]-1.0),1.0e-6,"grid");
  }
  bgrid.free();

  t.report();
  return 0;
}

