/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_GSL_SERIES_H
#define O2SCL_GSL_SERIES_H

#include <iostream>
#include <o2scl/collection.h>
#include <gsl/gsl_sum.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Series acceleration by Levin u-transform (GSL)

      Given an array of terms in a sum, this attempts to evaluate the
      entire sum with an estimate of the error.

      \todo Covert to use a more general vector

  */
  class gsl_series {
  public:
    
    /** \brief \c size is the number of terms in the series
     */
    gsl_series(int size=1);

    virtual ~gsl_series();

    /** \brief Return the accelerated sum of the series with
	a simple error estimate
     */
    double series_accel(double *x, double &err);
    
    /** \brief Return the accelerated sum of the series with
	an accurate error estimate
     */
    double series_accel_err(double *x, double &err);

    /** \brief Set the number of terms */
    int set_size(int new_size);
    
#ifndef DOXYGENP

    friend class io_tlate<gsl_series>;

  protected:

    /// The GSL workspace
    gsl_sum_levin_u_workspace *w;

    /// The GSL workspace
    gsl_sum_levin_utrunc_workspace *wt;

    /// The workspace size
    int size;

#endif

  };

  template<> int io_tlate<gsl_series>::input
    (cinput *co, in_file_format *ins,  gsl_series *gs);
  template<> int io_tlate<gsl_series>::output
    (coutput *co, out_file_format *outs, gsl_series *gs);
  template<> const char *io_tlate<gsl_series>::type();

  typedef io_tlate<gsl_series> gsl_series_io_type;

#ifndef DOXYGENP
}
#endif

#endif
