/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/vec_stats.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/ovector_cx_tlate.h>
#include <o2scl/omatrix_tlate.h>
#include <o2scl/omatrix_cx_tlate.h>
#include <o2scl/columnify.h>
#include <o2scl/array.h>
#include <gsl/gsl_statistics.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  cout.setf(ios::scientific);

  double x[10]={1,2,3,4,5,6,4,3,4,5};
  
  t.test_rel(vector_max<double [10]>(10,x),
	     gsl_stats_max(x,1,10),1.0e-8,"");
  t.test_rel(vector_min<double [10]>(10,x),
	     gsl_stats_min(x,1,10),1.0e-8,"");
  t.test_rel(vector_sum<double [10]>(10,x),
	     37.0,1.0e-8,"");
  t.test_rel(vector_mean<double [10]>(10,x),
	     gsl_stats_mean(x,1,10),1.0e-8,"");

  t.test_rel(vector_variance<double [10]>(10,x),
	     gsl_stats_variance(x,1,10),1.0e-8,"");
  t.test_rel(vector_variance<double [10]>(10,x,3.7),
	     gsl_stats_variance_m(x,1,10,3.7),1.0e-8,"");
  t.test_rel(vector_variance_fmean<double [10]>(10,x,3.7),
	     gsl_stats_variance_with_fixed_mean(x,1,10,3.7),1.0e-8,"");

  double sdtmp=vector_stddev<double [10]>(10,x);
  t.test_rel(vector_stddev<double [10]>(10,x),
	     gsl_stats_sd(x,1,10),1.0e-8,"");
  t.test_rel(vector_stddev<double [10]>(10,x,3.7),
	     gsl_stats_sd_m(x,1,10,3.7),1.0e-8,"");
  t.test_rel(vector_stddev_fmean<double [10]>(10,x,3.7),
	     gsl_stats_sd_with_fixed_mean(x,1,10,3.7),1.0e-8,"");

  t.test_rel(vector_absdev<double [10]>(10,x),
	     gsl_stats_absdev(x,1,10),1.0e-8,"");
  t.test_rel(vector_absdev<double [10]>(10,x,3.7),
	     gsl_stats_absdev_m(x,1,10,3.7),1.0e-8,"");
    
  t.test_rel(vector_skew<double [10]>(10,x),
	     gsl_stats_skew(x,1,10),1.0e-8,"");
  t.test_rel(vector_skew<double [10]>(10,x,3.7,sdtmp),
	     gsl_stats_skew_m_sd(x,1,10,3.7,sdtmp),1.0e-8,"");
  
  t.test_rel(vector_kurtosis<double [10]>(10,x),
	     gsl_stats_kurtosis(x,1,10),1.0e-8,"");
  t.test_rel(vector_kurtosis<double [10]>(10,x,3.7,sdtmp),
	     gsl_stats_kurtosis_m_sd(x,1,10,3.7,sdtmp),1.0e-8,"");
  
  t.test_rel(vector_lag1_autocorr<double [10]>(10,x),
	     gsl_stats_lag1_autocorrelation(x,1,10),1.0e-8,"");
  t.test_rel(vector_lag1_autocorr<double [10]>(10,x,3.7),
	     gsl_stats_lag1_autocorrelation_m(x,1,10,3.7),1.0e-8,"");
  
  vector_sort<double,double [10]>(10,x);
  t.test_rel(vector_median_sorted<double [10]>(10,x),
	     gsl_stats_median_from_sorted_data(x,1,10),1.0e-8,"");
  
  t.report();
  
  return 0;
}

