/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/classical.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

//--------------------------------------------
// classical class

classical::classical(double mass, double dof) : 
  part(mass,dof) {
  name="classical";
}

int classical::calc_mu(const double temper) {
  if (non_interacting==true) {nu=mu; ms=m;}
  if (inc_rest_mass) {
    if ((nu-m)/temper<-500.0) n=0.0;
    else n=exp((nu-m)/temper)*g*pow(ms*temper/pi/2.0,1.5);
    ed=1.5*temper*n+n*m;
  } else {
    if (nu/temper<-500.0) n=0.0;
    else n=exp(nu/temper)*g*pow(ms*temper/pi/2.0,1.5);
    ed=1.5*temper*n;
  }
  pr=n*temper;
  en=(ed+pr-n*nu)/temper;
  return 0;
}

int classical::calc_density(const double temper) {

  if (non_interacting==true) {ms=m;}
  
  if (n<=0.0) {
    if (inc_rest_mass) {
      nu=m;
    } else {
      nu=0.0;
    }
    if (non_interacting==true) {mu=nu;}
    ed=0.0;
    pr=0.0;
    en=0.0;
    if (n<0.0) return -1;
    else return 0;
  }
  
  if (inc_rest_mass) {
    nu=m+temper*log(n/g*pow(2.0*pi/ms/temper,1.5));
    ed=1.5*temper*n+n*m;
  } else {
    nu=temper*log(n/g*pow(2.0*pi/ms/temper,1.5));
    ed=1.5*temper*n;
  }

  if (non_interacting==true) {mu=nu;}

  pr=n*temper;
  en=(ed+pr-n*nu)/temper;
  return 0;
}

int classical::pair_mu(const double temper) {
  return calc_mu(temper);
}

int classical::pair_density(const double temper) {
  return calc_density(temper);
}

//--------------------------------------------
// classical_io 

template<> int io_tlate<classical>::input
(cinput *co, in_file_format *ins, classical *c) {
  part *p=(part *)c;
  std::string t;
  co->object_in("part",ins,p,t);
  return 0;
}

template<> int io_tlate<classical>::output
(coutput *co, out_file_format *outs, classical *c) {
  part *p=(part *)c;
  std::string t;
  co->object_out("part",outs,p,0,0,t);
  return 0;
}

template<> const char *io_tlate<classical>::type()
{ return "classical"; }
  
