/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_CLASSICAL_H
#define O2SCL_CLASSICAL_H

#include <string>
#include <iostream>
#include <fstream>
#include <cmath>
#include <o2scl/constants.h>
#include <o2scl/mroot.h>
#include <o2scl/inte.h>
#include <o2scl/part.h>
#include <o2scl/fermion.h>
#include <o2scl/boson.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Classical particle class

      Classical particles, because they inherit from both \ref fermion
      and \ref boson, have a fermi momentum, and a condensate, but
      these variables are ignored.
  */
  class classical : public fermion, public boson {

  public:

    /** \brief Create a classical particle with mass \c m  
	and degeneracy \c g 
    */
    classical(double m=0.0, double g=0.0);

    virtual int calc_mu(const double temper);
    virtual int calc_density(const double temper);
    virtual int pair_mu(const double temper);
    virtual int pair_density(const double temper);

    /// Return string denoting type ("classical")
    virtual const char *type() { return "classical"; }

#ifndef DOXYGEN_INTERNAL

  protected:
    
    friend class io_tlate<classical>;

#endif

  };

  template<> int io_tlate<classical>::input
    (cinput *co, in_file_format *ins, classical *c);
  template<> int io_tlate<classical>::output
    (coutput *co, out_file_format *outs, classical *c);
  template<> const char *io_tlate<classical>::type();
  
  typedef io_tlate<classical> classical_io_type;

#ifndef DOXYGENP
}
#endif

#endif
