/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/eff_fermion.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

// Static variables

double **eff_fermion::Pmnf=NULL, eff_fermion::parma=0.4;
int eff_fermion::sizem=4, eff_fermion::sizen=4;

eff_fermion::eff_fermion(double mass, double dof) : 
  part(mass,dof),fermion(mass,dof) {
  name="eff_fermion";

  tlimit=1.0e-8;

  if (Pmnf==NULL) {
    
    parma=0.42;
    Pmnf=new double *[4];
    for(size_t ip=0;ip<4;ip++) Pmnf[ip]=new double[4];
    Pmnf[0][0]=5.51219;
    Pmnf[0][1]=18.6215;
    Pmnf[0][2]=22.0078;
    Pmnf[0][3]=8.71963;
    Pmnf[1][0]=17.4343;
    Pmnf[1][1]=57.7188;
    Pmnf[1][2]=66.1098;
    Pmnf[1][3]=25.6323;
    Pmnf[2][0]=18.1239;
    Pmnf[2][1]=58.7781;
    Pmnf[2][2]=65.1429;
    Pmnf[2][3]=24.4246;
    Pmnf[3][0]=6.30952;
    Pmnf[3][1]=19.8967;
    Pmnf[3][2]=21.1375;
    Pmnf[3][3]=7.55866;
    sizem=3;
    sizen=3;
  }
  
  psi_root=&def_psi_root;
  density_root=&def_density_root;

}

eff_fermion::~eff_fermion() {
}

int eff_fermion::load_coefficients(int ctype) {
  
  if (ctype==cf_fermilat3) {
    delete_2d_array(Pmnf,sizem);
    sizem=4;
    sizen=4;
    Pmnf=new_2d_array<double>(sizem,sizen);
    Pmnf[0][0]=5.447327;
    Pmnf[0][1]=18.40237;
    Pmnf[0][2]=21.74882;
    Pmnf[0][3]=8.692826;
    Pmnf[1][0]=17.20333;
    Pmnf[1][1]=56.90495;
    Pmnf[1][2]=65.12704;
    Pmnf[1][3]=25.23724;
    Pmnf[2][0]=17.86753;
    Pmnf[2][1]=57.84077;
    Pmnf[2][2]=63.99390;
    Pmnf[2][3]=23.96116;
    Pmnf[3][0]=6.216856;
    Pmnf[3][1]=19.54390;
    Pmnf[3][2]=20.69990;
    Pmnf[3][3]=7.381855;
    parma=0.425;
  } else if (ctype==cf_fermijel2) {
    delete_2d_array(Pmnf,sizem);
    sizem=3;
    sizen=3;
    Pmnf=new_2d_array<double>(sizem,sizen);
    Pmnf[0][0]=5.23810;
    Pmnf[0][1]=12.4991;
    Pmnf[0][2]=8.36058;
    Pmnf[1][0]=11.1751;
    Pmnf[1][1]=25.3687;
    Pmnf[1][2]=15.6543;
    Pmnf[2][0]=5.91800;
    Pmnf[2][1]=12.4945;
    Pmnf[2][2]=6.82530;
    parma=0.442;
  } else if (ctype==cf_fermijel3) {
    delete_2d_array(Pmnf,sizem);
    sizem=4;
    sizen=4;
    Pmnf=new_2d_array<double>(sizem,sizen);
    Pmnf[0][0]=5.51219;
    Pmnf[0][1]=18.6215;
    Pmnf[0][2]=22.0078;
    Pmnf[0][3]=8.71963;
    Pmnf[1][0]=17.4343;
    Pmnf[1][1]=57.7188;
    Pmnf[1][2]=66.1098;
    Pmnf[1][3]=25.6323;
    Pmnf[2][0]=18.1239;
    Pmnf[2][1]=58.7781;
    Pmnf[2][2]=65.1429;
    Pmnf[2][3]=24.4246;
    Pmnf[3][0]=6.30952;
    Pmnf[3][1]=19.8967;
    Pmnf[3][2]=21.1375;
    Pmnf[3][3]=7.55866;
    parma=0.42;
  } else if (ctype==cf_fermijel3cons) {
    delete_2d_array(Pmnf,sizem);
    sizem=4;
    sizen=4;
    Pmnf=new_2d_array<double>(sizem,sizen);
    Pmnf[0][0]=5.34599;
    Pmnf[0][1]=18.0517;
    Pmnf[0][2]=21.3422;
    Pmnf[0][3]=8.53240;
    Pmnf[1][0]=16.8441;
    Pmnf[1][1]=55.7051;
    Pmnf[1][2]=63.6901;
    Pmnf[1][3]=24.6213;
    Pmnf[2][0]=17.4708;
    Pmnf[2][1]=56.3902;
    Pmnf[2][2]=62.1319;
    Pmnf[2][3]=23.2602;
    Pmnf[3][0]=6.07364;
    Pmnf[3][1]=18.9992;
    Pmnf[3][2]=20.0285;
    Pmnf[3][3]=7.11153;
    parma=0.433;
  } else {
    set_err_ret("Invalid type in eff_fermion::load_coefficients().",
		gsl_efailed);
  }

  return 0;
}

int eff_fermion::calc_mu(const double temper) {
  int nn, mm, ret=0;
  double xx, pren, preu, prep, sumn, sumu, sump;
  double f, gg, opf, opg, nc, psi;

  if (non_interacting) { nu=mu; ms=m; }

  if (ms<=0.0) {
    set_err_ret("Effective mass <= 0.0 in calc_mu().",gsl_einval);
  }

  if (temper<=tlimit) return fermion::calc_mu_zerot();
  
  // The following code assumes that the chemical potential
  // includes the rest mass, so we add it back in if necessary,
  // and we'll subtract it back out before finishing.
  if (!inc_rest_mass) nu+=m;

  funct_mfptr<eff_fermion,void *> mfs(this,&eff_fermion::solve_fun);
  
  int retx=0;
  
  // This section was revamped 3/03
  // using expansions in f of the equation
  // in solve_fun. 
  psi=(nu-ms)/temper;
  void *vp=(void *)&psi;
  if (psi<-200.0) {
    n=0.0;
    ed=0.0;
    pr=0.0;
    en=0.0;
    if (!inc_rest_mass) nu-=m;
    set_err_ret("(nu-ms)/T<-200.0 in eff_fermion::calc_mu().",gsl_einval);
  } else {
    if (psi<0.0) {
      // small f expansion
      xx=4.0*parma*exp(-2.0+psi);
    } else {
      // large f expansion
      xx=parma/8.0*(4.0+psi*psi+psi*sqrt(8.0+psi*psi));
    }
    retx=psi_root->solve(xx,vp,mfs);
    if (retx!=0) {
      add_err("Psi solver failed in eff_fermion::calc_mu(). Continuing.",
	      retx);
    }
    f=xx;
  }

  opf=1.0+f;
  gg=temper/ms*sqrt(opf);
  if (gg<=0.0) gg=0.0;
  opg=1.0+gg;
  nc=1.0/pi2*pow(ms,3.0);
    
  pren=f*pow(gg,1.5)/sqrt(1+f/parma)/pow(opf,sizem+0.5)/pow(opg,sizen-1.5);
  preu=f*pow(gg,2.5)*pow(opg,1.5)/pow(opf,sizem+1.0)/pow(opg,(double)sizen);
  prep=f*pow(gg,2.5)*pow(opg,1.5)/pow(opf,sizem+1.0)/pow(opg,(double)sizen);
  
  sumn=0.0;
  sumu=0.0;
  sump=0.0;
  for(mm=0;mm<=sizem;mm++) {
    for(nn=0;nn<=sizen;nn++) {
      sumn+=Pmnf[mm][nn]*pow(f,(double)mm)*pow(gg,(double)nn)*
	(1.0+mm+f/opf*(0.25+nn/2.0-sizem)+f*gg/opf/opg*(0.75-sizen/2.0));
      sumu+=Pmnf[mm][nn]*pow(f,(double)mm)*pow(gg,(double)nn)*
	(1.5+nn+gg/opg*(1.5-sizen));
      sump+=Pmnf[mm][nn]*pow(f,(double)mm)*pow(gg,(double)nn);
    }
  } 

  n=g/2.0*pren*sumn*nc;
  pr=g/2.0*prep*sump*nc*ms;

  // Note that here, we are adding the rest mass contribution with
  // the effective mass, since the EFF coefficients, even though
  // they are fully relativistic, were designed to compute the
  // energy without the rest mass term. The rest mass energy is
  // removed later if necessary.
  ed=g/2.0*preu*sumu*nc*ms+n*ms;

  en=(ed+pr-n*nu)/temper;

  if (!inc_rest_mass) {
    nu-=m;
    ed-=n*m;
  }
  
  // Return an error if psi_root failed.
  return retx;
}

int eff_fermion::calc_density(const double temper) {
  double xx, f, sqt, aarg, opf, gg, opg, nc, T=temper;
  double pren, preu, prep, sumn, sumu, sump, psi;
  int mm, nn;

  if (non_interacting) { ms=m; nu=mu;}
  if (temper<=tlimit) return fermion::calc_density_zerot();
  
  if (n==0.0) {
    nu=ms;
    if (!inc_rest_mass) nu=0.0;
    ed=0.0;
    pr=0.0;
    en=0.0;
    if (non_interacting) { mu=nu; }
    return 0;
  }
  
  nc=1.0/pi2*pow(ms,3.0);
  
  funct_mfptr<eff_fermion,void *> mfd(this,&eff_fermion::density_fun);
  double newx=nu, tmp;
  void *vp=(void *)&T;
  int ret=density_root->solve(newx,vp,mfd);
  density_fun(newx,tmp,vp);
  if (ret!=0) {
    add_err("Density failed in eff_fermion::calc_density(). Continuing.",
	    ret);
  }

  nu=newx;

  if (non_interacting) mu=nu;

  return 0;
}

int eff_fermion::solve_fun(double x, double &y, void *&pa) {
  double f, sqt, aarg, psi;

  psi=*((double *)pa);
  f=x;

  if (f<=0.0) {
    set_err("f<0 in eff_fermion::solve_fun().",2);
    return 2;
  }

  // For small f, we lose too much precision using
  // the usual formula, so we use the expansion
  if (f/parma<1.0e-10) {
    sqt=1.0+f/parma/2.0;
    aarg=f/parma/2.0/(sqt+1.0);
  } else {
    sqt=sqrt(1.0+f/parma);
    aarg=(sqt-1.0)/(sqt+1.0);
  }
  
  if (fabs(psi)<1.0e-10) {
    y=(2.0*sqt+log(aarg))-psi;
  } else {
    y=(2.0*sqt+log(aarg))/psi-1.0;
  }

  if (!finite(y)) {
    set_err("y not finite in eff_fermion::solve_fun().",1);
    return 1;
  }

  return 0;
}

int eff_fermion::density_fun(double x, double &y, void *&pa) {
  double *temper=(double *)pa;
  double nold, nnew;

  if (non_interacting) mu=x;
  else nu=x;
  nold=n;
  calc_mu(*temper);
  nnew=n;
  y=nnew/nold-1.0;
  n=nold;

  return 0;
}

// Finite temperature massive fermions and antifermions (uses nu, mu and ms)
int eff_fermion::pair_mu(const double temper) {
  if (non_interacting) { nu=mu; ms=m; }

  int ret;

  if (ms==0.0) {

    ret=massless_pair_mu(temper);

  } else {

    eff_fermion antip(ms,g);
    anti(antip);

    antip.massless_root=massless_root;
    antip.psi_root=psi_root;
    antip.density_root=density_root;
    
    ret=calc_mu(temper);
    if (ret!=0) {
      add_err("Function calc_mu() failed in eff_fermion::pair_mu().",ret);
    }
    
    ret=antip.calc_mu(temper);
    if (ret!=0) {
      add_err("calc_mu() failed for antiparticles eff_fermion::pair_mu().",
	      ret);
    }

    n-=antip.n;
    pr+=antip.pr;
    ed+=antip.ed;
    en+=antip.en;
  }
  
  return ret;
}

int eff_fermion::pair_density(const double temper) {
  double f, sqt, aarg, opf, gg, opg, sumn, sumu, sump;
  double pren, preu, prep, nc, psi, T=temper;
  int mm, nn;
  ovector xx(2);

  if (non_interacting) { ms=m; nu=mu; }
  if (ms==0.0) return massless_pair_density(temper);
  
  funct_mfptr<eff_fermion,void *> mfd(this,&eff_fermion::pair_density_fun);
  
  xx[0]=nu;
  void *vpx=(void *)&T;
  double tmp;
  int ret=density_root->solve(xx[0],vpx,mfd);
  pair_density_fun(xx[0],tmp,vpx);
  if (ret!=0) {
    add_err("mroot failed in pair_density().",ret);
  }
  nu=xx[0];
  
  if (non_interacting) { mu=nu; }

  return ret;
}

int eff_fermion::pair_density_fun(double x, double &y, void *&pa) {
  double *temper=(double *)pa, nold, nnew;
  
  if (non_interacting) mu=x;
  else nu=x;
  nold=n;
  pair_mu(*temper);
  nnew=n;
  y=nnew/nold-1.0;
  n=nold;

  return 0;
}

template<> int io_tlate<eff_fermion>::input(cinput *co, in_file_format *ins, 
					    eff_fermion *f) {
  fermion *g=(fermion *)f;
  std::string t;
  co->object_in("fermion",ins,g,t);
  return 0;
}

template<> int io_tlate<eff_fermion>::output(coutput *co, 
					     out_file_format *outs, 
					     eff_fermion *f) {
  fermion *g=(fermion *)f;
  std::string t;
  co->object_out("fermion",outs,g,0,0,t);
  return 0;
}

template<> const char *io_tlate<eff_fermion>::type() { return "eff_fermion"; }

