/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/eff_quark.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

//--------------------------------------------
// eff_quark class

eff_quark::eff_quark(double mass, double dof) : 
  part(mass,dof),fermion(mass,dof),eff_fermion(mass,dof),quark(mass,dof) {
  B=0.0;
  qq=0.0;
  name="eff_quark";
}

eff_quark::~eff_quark() {}

int eff_quark::calc_mu(const double temper) {
  return eff_fermion::calc_mu(temper);
}

int eff_quark::calc_density(const double temper) {
  return eff_fermion::calc_density(temper);
}

int eff_quark::pair_mu(const double temper) {
  return eff_fermion::pair_mu(temper);
}

int eff_quark::pair_density(const double temper) {
  return eff_fermion::pair_density(temper);
}

template<> int io_tlate<eff_quark>::input
(cinput *co, in_file_format *ins, eff_quark *q) {
  eff_fermion *f=(eff_fermion *)q;
  std::string t;
  co->object_in("eff_fermion",ins,f,t);
  ins->double_in(q->B,"B");
  ins->double_in(q->qq,"qq");
  return 0;
}

template<> int io_tlate<eff_quark>::output
(coutput *co, out_file_format *outs, eff_quark *q) {
  eff_fermion *f=(eff_fermion *)q;
  std::string t;
  co->object_out("eff_fermion",outs,f,0,0,t);
  outs->double_out(q->B,"B");
  outs->double_out(q->qq,"qq");
  return 0;
}

template<> const char *io_tlate<eff_quark>::type()
{ return "eff_quark"; }
