/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/gsl_rnga.h>

using namespace std;
using namespace o2scl;

#include <iostream>
#include <o2scl/test_mgr.h>

#define N 20000

class one {
public:
  gsl_rnga gr;
  
  void fun() {
    for(int i=0;i<N;i++) {
      for(int j=0;j<N;j++) {
	gr.random();
      }
    }
    return;
  }
};

template<class rng_t> class two {
 public:

  rng_t gr;

  void fun() {
    for(int i=0;i<N;i++) {
      for(int j=0;j<N;j++) {
	gr.random();
      }
    }
    return;
  }

};

int main(void) {
  double a1, a2;
  test_mgr t;
  t.set_output_level(2);

  gsl_rnga nr(10);

  a1=nr.random();
  a2=nr.random();

  gsl_rnga nr2(10);

  // Test to make sure that given the same seed, we 
  // come up with the same numbers.
  t.test_rel(a1,nr2.random(),1.0e-14,"First random number.");
  t.test_rel(a2,nr2.random(),1.0e-14,"Second random number.");

  cout << "Random integers [0,10): " << flush;
  for(int i=0;i<10;i++) {
    cout << nr2.random_int(10) << " " << flush;
  }
  cout << endl;

  /*
    gsl_rng *rp;
    rp=nr2.get_gsl_rng();
    cout << rp->type->size << " " << sizeof(char) << endl;
    char *ip=(char *)rp->state;
    for(int j=0;j<((int)rp->type->size)/10;j++) {
    cout << ((int)ip[j]) << " " << ip[j] << " " << endl;;
    }
    cout << endl;
    exit(-1);
  */

  /*
    gsl_rnga *nr4;
    collection co;
    void *vp;
    co.test_type(t,"gsl_rnga",&nr,vp,1);
    nr4=(gsl_rnga *)vp;
    t.test_gen(nr.get_seed()==nr4->get_seed(),"params test");
  */

  one t1;
  two<gsl_rnga> t2;

  cout << clock() << endl;
  t1.fun();
  cout << clock() << endl;
  t2.fun();
  cout << clock() << endl;

  t.report();
  return 0;
}
