/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#ifndef O2SCL_GSL_MROOT_HYBRIDS_B_H
#define O2SCL_GSL_MROOT_HYBRIDS_B_H

#include <cmath>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Base functions for gsl_mroot_hybrids

      This is a trivial recasting of the functions that were in file
      scope in the GSL version of the hybrids solver. 
      
      \todo Document the individual functions for this class
  
  */
  class hybrids_base {
  protected:

    /// Compute the norm of \c f
    double enorm(const gsl_vector *f);

    /// Compute the norm of \f$ \vec{f} \cdot \vec{d} \f$
    double scaled_enorm(const gsl_vector *d, const gsl_vector *f);

    /// Compute the norm of \f$ \vec{a} + \vec{b} \f$.
    double enorm_sum(const gsl_vector *a, const gsl_vector *b);
  
    /// Desc.
    void compute_wv(const gsl_vector *qtdf, const gsl_vector *rdx, 
		    const gsl_vector *dx, const gsl_vector *diag, 
		    double pnorm, gsl_vector *w, gsl_vector *v);
    
    /// Desc.
    void compute_df(const gsl_vector *f_trial, const gsl_vector *f, 
		    gsl_vector *df);
    
    /// Desc.
    void compute_diag(const gsl_matrix *J, gsl_vector *diag);

    /// Desc.
    void update_diag(const gsl_matrix *J, gsl_vector *diag);

    /// Desc.
    double compute_delta(gsl_vector *diag, gsl_vector *x);

    /// Desc.
    double compute_actual_reduction(double fnorm, double fnorm1);

    /// Desc.
    double compute_predicted_reduction(double fnorm, double fnorm1);

    /// Compute \f$ Q^{T} f \f$
    void compute_qtf(const gsl_matrix *q, const gsl_vector *f, 
		     gsl_vector *qtf);

    /// Desc.
    void compute_rdx(const gsl_matrix *r, const gsl_vector *dx, 
		     gsl_vector *rdx);

    /// Desc.
    void compute_trial_step(gsl_vector *x, gsl_vector *dx, 
			    gsl_vector *x_trial);

    /// Desc.
    int newton_direction(const gsl_matrix *r, const gsl_vector *qtf, 
			 gsl_vector *p);

    /// Desc.
    void gradient_direction(const gsl_matrix *r, const gsl_vector *qtf,
			    const gsl_vector *diag, gsl_vector *g);

    /// Desc.
    void minimum_step(double gnorm, const gsl_vector *diag, gsl_vector *g);


    /// Desc.
    void compute_Rg(const gsl_matrix *r, const gsl_vector *gradient, 
		    gsl_vector *Rg);

    /// Desc.
    void scaled_addition(double alpha, gsl_vector *newton, double beta, 
			 gsl_vector *gradient, gsl_vector *p);

    /// Take a dogleg step
    int dogleg(const gsl_matrix *r, const gsl_vector *qtf,
	       const gsl_vector *diag, double delta,
	       gsl_vector *newton, gsl_vector *gradient, gsl_vector *p);

  };
  
#ifndef DOXYGENP
}
#endif

#endif
